/* PPLite: a lightweight library for convex polyhedra derived from PPL.
   Copyright (C) 2018 Enea Zaffanella <enea.zaffanella@unipr.it>

This file is part of PPLite.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pplite_test.hh"

bool
test01() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A > 4);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A > 4);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A <= 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test02() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A > 2);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A > 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A <= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test03() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A <= 0);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A <= 0);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A > 0);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test04() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A <= 2);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A <= 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A > 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test05() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2);
  ph1.add_cons(cs);

  Con c(A >= 4);

  Poly ph2 = ph1.split(c);

  Poly knr1(2);
  knr1.add_cons(cs);
  knr1.add_con(A >= 4);
  knr1.minimize();

  Poly knr2(2);
  knr2.add_cons(cs);
  knr2.add_con(A <= 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test06() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2);
  ph1.add_cons(cs);

  Con c(A >= 2);

  Poly ph2 = ph1.split(c);

  Poly knr1(2);
  knr1.add_cons(cs);
  knr1.add_con(A >= 2);
  knr1.minimize();

  Poly knr2(2);
  knr2.add_cons(cs);
  knr2.add_con(A <= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test07() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2);
  ph1.add_cons(cs);

  Con c(A <= 0);

  Poly ph2 = ph1.split(c);

  Poly knr1(2);
  knr1.add_cons(cs);
  knr1.add_con(A <= 0);
  knr1.minimize();

  Poly knr2(2);
  knr2.add_cons(cs);
  knr2.add_con(A >= 0);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test08() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A <= 4);
  cs.push_back(B <= 4);

  Poly ph1(2);
  ph1.add_cons(cs);

  Con c(A <= 2);

  Poly ph2 = ph1.split(c);

  Poly knr1(2);
  knr1.add_cons(cs);
  knr1.add_con(A <= 2);
  knr1.minimize();

  Poly knr2(2);
  knr2.add_cons(cs);
  knr2.add_con(A >= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test09() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B > 0);
  cs.push_back(A <= 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(B <= 2);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(B <= 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(B > 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test10() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A > 0);
  cs.push_back(B > 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(B < 2);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(B < 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(B >= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test11() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B <= 4);
  cs.push_back(A + B > 0);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(B > 2);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(B > 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(B <= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test12() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A + B <= 4);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A + B <= 4);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A + B > 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test13() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A + B <= 0);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A + B <= 0);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A + B > 0);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test14() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A + B > 4);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A + B > 4);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A + B <= 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test15() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(B >= 0);
  cs.push_back(A < 4);
  cs.push_back(B < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A < 4);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A < 4);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A >= 4);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test16() {
  Var A(0);
  Var B(1);
  Var C(2);
  Var D(3);

  Poly ph;
  std::string input = R"###(topol NNC
dim 4
status MINIMIZED
=> cs sys
sing_rows 0
sk_rows 16
> : dim 4 : -1 -1 1 1  : 4
> : dim 4 : -1 1 -1 1  : 4
> : dim 4 : -1 1 1 1  : 0
> : dim 4 : -1 1 1 -1  : 4
> : dim 4 : 1 -1 -1 1  : 4
> : dim 4 : 1 -1 1 1  : 0
> : dim 4 : 1 -1 1 -1  : 4
> : dim 4 : 1 1 -1 1  : 0
>= : dim 4 : 1 1 -1 -1  : 4
> : dim 4 : 1 1 1 1  : -4
> : dim 4 : 1 1 1 -1  : 0
>= : dim 4 : -1 -1 -1 -1  : 9
> : dim 4 : -1 -1 -1 1  : 7
> : dim 4 : -1 -1 1 -1  : 7
>= : dim 4 : -1 1 -1 -1  : 7
>= : dim 4 : 1 -1 -1 -1  : 7
ns_rows 0
=> gs sys
sing_rows 0
sk_rows 36
C : dim 4 : 4 4 3 -3  : 2
C : dim 4 : 4 3 4 -3  : 2
C : dim 4 : 3 4 4 -3  : 2
C : dim 4 : 4 3 9 2  : 2
C : dim 4 : 3 4 9 2  : 2
C : dim 4 : 4 9 3 2  : 2
C : dim 4 : 3 9 4 2  : 2
C : dim 4 : 9 4 3 2  : 2
C : dim 4 : 9 3 4 2  : 2
C : dim 4 : 4 3 -3 4  : 2
C : dim 4 : 3 4 -3 4  : 2
C : dim 4 : 4 3 2 9  : 2
C : dim 4 : 3 4 2 9  : 2
C : dim 4 : 3 9 2 4  : 2
C : dim 4 : 9 3 2 4  : 2
C : dim 4 : 4 4 -3 3  : 2
C : dim 4 : 4 9 2 3  : 2
C : dim 4 : 9 4 2 3  : 2
C : dim 4 : 3 -3 4 4  : 2
C : dim 4 : 4 -3 4 3  : 2
C : dim 4 : 4 -3 3 4  : 2
C : dim 4 : 3 2 4 9  : 2
C : dim 4 : 4 2 3 9  : 2
C : dim 4 : 3 2 9 4  : 2
C : dim 4 : 4 2 9 3  : 2
C : dim 4 : 9 2 4 3  : 2
C : dim 4 : 9 2 3 4  : 2
C : dim 4 : -3 4 4 3  : 2
C : dim 4 : -3 4 3 4  : 2
C : dim 4 : -3 3 4 4  : 2
C : dim 4 : 2 4 3 9  : 2
C : dim 4 : 2 3 4 9  : 2
C : dim 4 : 2 4 9 3  : 2
C : dim 4 : 2 3 9 4  : 2
C : dim 4 : 2 9 4 3  : 2
C : dim 4 : 2 9 3 4  : 2
ns_rows 2
2 : { 21, 23 }
2 : { 31, 33 }
sat_c
36 x 16
0 1 0 1 1 0 1 1 1 0 1 1 0 1 1 1 
1 0 0 1 1 1 1 0 1 0 1 1 0 1 1 1 
1 1 1 1 0 0 1 0 1 0 1 1 0 1 1 1 
1 0 1 1 1 1 1 0 1 1 1 0 0 1 1 1 
1 1 1 1 0 1 1 0 1 1 1 0 0 1 1 1 
0 1 1 1 1 0 1 1 1 1 1 0 0 1 1 1 
1 1 1 1 0 0 1 1 1 1 1 0 0 1 1 1 
0 1 0 1 1 1 1 1 1 1 1 0 0 1 1 1 
1 0 0 1 1 1 1 1 1 1 1 0 0 1 1 1 
1 1 0 0 1 1 1 1 1 0 0 1 1 0 1 1 
1 1 1 1 1 0 0 1 1 0 0 1 1 0 1 1 
1 1 1 0 1 1 1 1 1 1 0 0 1 0 1 1 
1 1 1 1 1 1 0 1 1 1 0 0 1 0 1 1 
1 1 1 1 1 0 0 1 1 1 1 0 1 0 1 1 
1 1 0 0 1 1 1 1 1 1 1 0 1 0 1 1 
0 1 0 1 1 0 1 1 1 0 1 1 1 0 1 1 
0 1 1 1 1 0 1 1 1 1 1 0 1 0 1 1 
0 1 0 1 1 1 1 1 1 1 1 0 1 0 1 1 
1 1 1 1 1 1 1 0 0 0 0 1 1 1 0 1 
1 0 0 1 1 1 1 0 1 0 1 1 1 1 0 1 
1 1 0 0 1 1 1 1 1 0 0 1 1 1 0 1 
1 1 1 1 1 1 1 1 0 1 0 0 1 1 0 1 
1 1 1 0 1 1 1 1 1 1 0 0 1 1 0 1 
1 1 1 1 1 1 1 0 0 1 1 0 1 1 0 1 
1 0 1 1 1 1 1 0 1 1 1 0 1 1 0 1 
1 0 0 1 1 1 1 1 1 1 1 0 1 1 0 1 
1 1 0 0 1 1 1 1 1 1 1 0 1 1 0 1 
1 1 1 1 0 0 1 0 1 0 1 1 1 1 1 0 
1 1 1 1 1 0 0 1 1 0 0 1 1 1 1 0 
1 1 1 1 1 1 1 0 0 0 0 1 1 1 1 0 
1 1 1 1 1 1 0 1 1 1 0 0 1 1 1 0 
1 1 1 1 1 1 1 1 0 1 0 0 1 1 1 0 
1 1 1 1 0 1 1 0 1 1 1 0 1 1 1 0 
1 1 1 1 1 1 1 0 0 1 1 0 1 1 1 0 
1 1 1 1 0 0 1 1 1 1 1 0 1 1 1 0 
1 1 1 1 1 0 0 1 1 1 1 0 1 1 1 0 
sat_g
16 x 36
0 1 1 1 1 0 1 0 1 1 1 1 1 1 1 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 
1 0 1 0 1 1 1 1 0 1 1 1 1 1 1 1 1 1 1 0 1 1 1 1 0 0 1 1 1 1 1 1 1 1 1 1 
0 0 1 1 1 1 1 0 0 0 1 1 1 1 0 0 1 0 1 0 0 1 1 1 1 0 0 1 1 1 1 1 1 1 1 1 
1 1 1 1 1 1 1 1 1 0 1 0 1 1 0 1 1 1 1 1 0 1 0 1 1 1 0 1 1 1 1 1 1 1 1 1 
1 1 0 1 0 1 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 1 1 1 1 0 1 0 1 
0 1 0 1 1 0 0 1 1 1 0 1 1 0 1 0 0 1 1 1 1 1 1 1 1 1 1 0 0 1 1 1 1 1 0 0 
1 1 1 1 1 1 1 1 1 1 0 1 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 1 0 1 1 1 1 0 
1 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 1 1 1 0 0 1 1 0 1 0 1 1 0 0 1 1 
1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 1 1 0 1 0 1 1 1 1 1 0 1 0 1 0 1 1 
0 0 0 1 1 1 1 1 1 0 0 1 1 1 1 0 1 1 0 0 0 1 1 1 1 1 1 0 0 0 1 1 1 1 1 1 
1 1 1 1 1 1 1 1 1 0 0 0 0 1 1 1 1 1 0 1 0 0 0 1 1 1 1 1 0 0 0 0 1 1 1 1 
1 1 1 0 0 0 0 0 0 1 1 0 0 0 0 1 0 0 1 1 1 0 0 0 0 0 0 1 1 1 0 0 0 0 0 0 
0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 
1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 
1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 
1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0 0 0 0 0 0 0 0 0 
=> cs_pending
sing_rows 0
sk_rows 0
ns_rows 0
=> gs_pending
sing_rows 0
sk_rows 0
ns_rows 0
)###";
  std::stringstream ss_in(input);
  ph.ascii_load(ss_in);

  Con c(B + C + D < 8);

  Poly knres_then(ph);
  knres_then.add_con(c);
  Poly ph_else = ph.split(c);

  bool ok = (knres_then == ph);
  return ok;
}

bool
test17() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(A < 4);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(B <= 4);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(B <= 4);

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(B > 4);

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test18() {
  Var A(0);
  Var B(1);

  Poly ph1(2, Topol::NNC);

  Con c(A + B <= 4);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_con(A + B <= 4);

  Poly knr2(2, Topol::NNC);
  knr2.add_con(A + B > 4);

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test19() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A >= 0);
  cs.push_back(A <= 3);

  Poly ph1(2, Topol::CLOSED);
  ph1.add_cons(cs);

  Con c(B <= 0);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::CLOSED);
  knr1.add_cons(cs);
  knr1.add_con(B <= 0);
  knr1.minimize();

  Poly knr2(2, Topol::CLOSED);
  knr2.add_cons(cs);
  knr2.add_con(B >= 0);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

bool
test20() {
  Var A(0);
  Var B(1);
  Cons cs;
  cs.push_back(A > 0);
  cs.push_back(A < 3);

  Poly ph1(2, Topol::NNC);
  ph1.add_cons(cs);

  Con c(A < 2);

  Poly ph2 = ph1.split(c);

  Poly knr1(2, Topol::NNC);
  knr1.add_cons(cs);
  knr1.add_con(A < 2);
  knr1.minimize();

  Poly knr2(2, Topol::NNC);
  knr2.add_cons(cs);
  knr2.add_con(A >= 2);
  knr2.minimize();

  return (ph1 == knr1 && ph2 == knr2);
}

BEGIN_MAIN
  DO_TEST(test01);
  DO_TEST(test02);
  DO_TEST(test03);
  DO_TEST(test04);
  DO_TEST(test05);
  DO_TEST(test06);
  DO_TEST(test07);
  DO_TEST(test08);
  DO_TEST(test09);
  DO_TEST(test10);
  DO_TEST(test11);
  DO_TEST(test12);
  DO_TEST(test13);
  DO_TEST(test14);
  DO_TEST(test15);
  DO_TEST(test16);
  DO_TEST(test17);
  DO_TEST(test18);
  DO_TEST(test19);
  DO_TEST(test20);
END_MAIN

